from enum import StrEnum

from pydantic import BaseModel

from salute.common.models.message.payload import Payload


class Suggestion(BaseModel):
    buttons: list[dict] = []


class PronounceType(StrEnum):
    TEXT = "application/text"
    SSML = "application/ssml"

    def __str__(self):
        return self.value

    def __repr__(self):
        return self.value


class ResponsePayload(Payload):
    pronounceText: str = ""
    pronounceTextType: PronounceType = PronounceType.TEXT
    items: list = []
    suggestions: Suggestion = Suggestion()
    auto_listening: bool = False



import argparse
from salute.cli.actions import InitScenarioAction
from salute.cli.system import get_version


def command():
    parser = argparse.ArgumentParser(
        description="Salute CLI tool for managing your application"
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s {get_version()}",
        help="Show program's version number and exit",
    )
    parser.add_argument(
        "-i",
        "--init",
        action=InitScenarioAction,
        help="Execute init scenario to prepare your application",
        nargs=0,
    )

    args = parser.parse_args()

    if not any(vars(args).values()):
        parser.print_help()
        return



import toml
from pathlib import Path
from venv import logger


class SystemToolError(Exception):
    pass


def get_version() -> str:
    try:
        with Path("pyproject.toml").open("r") as f:
            toml_dict = toml.load(f)
            print(toml_dict)
            return toml_dict.get("project", {}).get("version", "0.0.0")
    except (FileNotFoundError, toml.TomlDecodeError, KeyError) as e:
        raise SystemToolError(f"Failed to get version: {e}")
    except Exception as e:
        raise SystemToolError(f"Unknown error to get version: {e}")


def set_version(current_version: str) -> None:
    try:
        with Path("pyproject.toml").open("r") as current_file:
            toml_dict = toml.load(current_file)
            toml_dict["project"]["version"] = current_version
            toml_dict.update({"project": toml_dict["project"]})

            with open("pyproject.toml", "w") as update_file:
                toml.dump(toml_dict, update_file)
    except (toml.TomlDecodeError, KeyError) as e:
        raise SystemToolError(
            f"Failed to update version {current_file} with error: {e}"
        )
    except FileNotFoundError:
        raise SystemToolError("Failed to update version: pyproject.toml not found")
    except PermissionError:
        raise SystemToolError("Failed to update version: Permission denied")
    except Exception as e:
        raise SystemToolError(
            f"Unknow error to update version {current_file} with error: {e}"
        )


def update_version():
    current_version = get_version()
    logger.info(f"Current version {current_version}")
    major, minor, patch = current_version.split(".")
    new_version = f"{major}.{minor}.{int(patch) + 1}"
    set_version(new_version)
    logger.info(f"New version is {new_version}")



from importlib.metadata import entry_points
from sanic.log import logger


def use_plugins():
    discovered_plugins = entry_points(group="salute.plugins")
    logger.info("Initializing plugins")

    for plugin in discovered_plugins:
        logger.info(f"Loading {plugin.name}")

        try:
            current_plugin = plugin.load()
            current_plugin()

            logger.info(f"Plugin {plugin.name} successfully loaded")

        except Exception as e:
            logger.error(f"Failed to load plugin {plugin.name}: {e}")



from argparse import Action
from pathlib import Path

from salute.utils.const import SCENARIO_SRC
from loguru import logger


class InitScenarioAction(Action):
    def __init__(self, option_strings, dest, **kwargs):
        super().__init__(option_strings=option_strings, dest=dest, **kwargs)
        self.path = Path().resolve()

    def __call__(self, parser, namespace, values, option_string=None):
        logger.info("Executing init scenario...")
        setattr(namespace, self.dest, True)
        self.create_scenario_folder()

    def create_scenario_folder(self):
        logger.info("Starting creating scenario...")

        scenario_folder = self.path / SCENARIO_SRC

        if scenario_folder.exists():
            logger.error("Scenario folder already exists")
            exit(1)

        try:
            scenario_folder.mkdir()
        except Exception as e:
            logger.error(f"Failed to create scenario folder: {e}")
            exit(2)

        logger.info("Scenario folder created successfully")
